"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.calcCalories = exports.carbonation = exports.yeastStarterGrow = exports.yeastCount = exports.yeastNeeded = void 0;
const utils_1 = require("./utils");
const yeast_1 = require("./types/yeast");
// https://www.brewersfriend.com/yeast-pitch-rate-and-starter-calculator/
// million cells / ml / degree Plato
// Minimum manufacturer's recommendation: 0.35 (ale only, fresh yeast only)
// Middle of the road Pro Brewer 0.75 (ale)
// Pro Brewer 1.00 (high gravity ale)
// Pro Brewer 1.50 (minimum for lager)
// Pro Brewer 2.0 (high gravity lager)
// cellDensity = billion cells / gram
// Safale K-97	14
// Safale S-04	8
// Safbrew T-58	18
// Safbrew S-33	16
// Saflager S-23	10
// Saflager S-189	9
// A pack/vial contains 100 billion cells at the date of manufacture.
// Liquid yeast viability drops 21% each month, or 0.7% each day, from the date of manufacture.
// The assumption is the yeast viability drops in a linear fashion. In 4.75 months or 143 days, this calculator assumes the yeast is 100% dead (100 / 0.7 = ~143).
// million 10 ^ 6
// billion 10 ^ 9
const yeastNeeded = (pitchRate, batchSize, e) => (pitchRate * (batchSize * 1000) * e) / 1000;
exports.yeastNeeded = yeastNeeded;
const viability = (currentDate, cultureDate = new Date().toString()) => 100 -
    Math.floor((Date.parse(currentDate) - Date.parse(cultureDate)) / 86400000) *
        0.7;
const yeastCount = ({ amount, form, cultureDate }, currentDate = new Date().toString(), cellDensity = 8, 
// billion cells / ml
slurryDensity = 1) => {
    switch (form) {
        case yeast_1.YeastForms.dry:
            return cellDensity * amount * 1000;
        case yeast_1.YeastForms.liquid:
            return 100 * (viability(currentDate, cultureDate) / 100) * amount;
        case yeast_1.YeastForms.slant:
            return slurryDensity * amount * 1000;
        default:
            throw new Error('NotImplementedError');
    }
};
exports.yeastCount = yeastCount;
const yeastGrowth = (ratio) => 2.33 - 0.67 * ratio;
const growthRateCurveBraukaiserStir = (ratio) => ratio < 1.4
    ? 1.4
    : ratio >= 1.4 && ratio <= 3.5 && yeastGrowth(ratio) > 0
        ? yeastGrowth(ratio)
        : 0;
const yeastStarterGrow = (startingYeastCount, starterSize, gravity, batchSize) => {
    const volumeLevel = utils_1.litersToGallons(starterSize);
    const pointsNeeded = volumeLevel * (gravity - 1) * 1000;
    const poundsDME = pointsNeeded / 42;
    const gramsDME = utils_1.poundsTokg(poundsDME) * 1000;
    const cellsToGramsRatio = startingYeastCount / gramsDME;
    const growthRate = growthRateCurveBraukaiserStir(cellsToGramsRatio);
    const endingCount = gramsDME * growthRate + startingYeastCount;
    const pitchRate = (endingCount * 1000) / utils_1.sgToPlato(gravity) / (batchSize / 1000);
    return {
        growthRate: growthRate,
        endingCount: endingCount,
        pitchRate: pitchRate,
    };
};
exports.yeastStarterGrow = yeastStarterGrow;
// https://byo.com/yeast/item/164-balancing-your-draft-system-advanced-brewing
const kegPressure = (carbVolume, t) => Math.max(0, -16.6999 -
    0.0101059 * t +
    0.00116512 * t * t +
    0.173354 * t * carbVolume +
    4.24267 * carbVolume -
    0.0684226 * carbVolume * carbVolume);
// http://www.homebrewtalk.com/showthread.php?t=441383
const primingSugar = (carbVolume, t, batchSize) => 15.195 * batchSize * (carbVolume - 3.0378 + 5.0062e-2 * t - 2.6555e-4 * t * t);
const normalizeTemp = (t) => Math.max(32.0, utils_1.celsiusToFahrenheit(t));
const carbonation = (carbVolume, t, batchSize) => {
    const sugar = primingSugar(carbVolume, normalizeTemp(t), utils_1.litersToGallons(batchSize));
    return {
        kegPressure: kegPressure(carbVolume, normalizeTemp(t)),
        kegSugar: sugar * 0.5,
        cornSugar: sugar,
        dme: sugar * 1.538,
    };
};
exports.carbonation = carbonation;
// http://beersmith.com/blog/2011/02/04/counting-calories-in-your-homebrewed-beer/
// Calorie_from_alcohol = 1881.22 * FG * (OG-FG)/(1.775-OG)
// Calories_from_carbs = 3550.0 * FG * ((0.1808 * OG) + (0.8192 * FG) – 1.0004)
// Total calories – just add the Calories_from_alcohol to Calories_from_carbs
const caloriesAlc = (og, fg) => 1881.22 * fg * ((og - fg) / (1.775 - og));
const caloriesExt = (og, fg) => 3550.0 * fg * (0.1808 * og + 0.8192 * fg - 1.0004);
const calcCalories = (og, fg) => caloriesAlc(og, fg) + caloriesExt(og, fg);
exports.calcCalories = calcCalories;
//# sourceMappingURL=brewcalc.js.map