"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.bitternessIbuRager = exports.bitternessRatio = exports.bitternessIbuTinseth = void 0;
const utils_1 = require("./utils");
const units_1 = require("./units");
const timing_1 = require("./timing");
const alphaAcidUnits = (amountInOz, alphaAcid) => amountInOz * alphaAcid;
const gravityFactor = (boilGravityValue) => 1.65 * Math.pow(0.000125, boilGravityValue - 1);
const timeFactor = (boilTimeInMin) => (1 - Math.exp(-0.04 * boilTimeInMin)) / 4.15;
const pelletFactor = (form = '') => form === 'pellet' ? 1.1 : 1;
const ibuUtilization = (hopForm = '', boilGravityValue, boilTimeInMin = 0) => pelletFactor(hopForm) *
    gravityFactor(boilGravityValue) *
    timeFactor(boilTimeInMin);
// Glenn Tinseth developed the following formula to calculate bitterness in IBUs:
// IBU = (U * ozs hops * 7490)/Volume (in gallons) U represents the utilization of the hops (conversion to iso-alpha-acids) based on boil time and wort gravity.
// U = bigness factor * boil time factor
// http://www.howtobrew.com/book/section-1/hops/hop-bittering-calculations
const bitternessIbuTinseth = (hops, boilGravity, postBoilVolume) => {
    const bitterness = utils_1.sum(hops.map(({ amount, alpha_acid, form, timing }) => {
        // TODO: research needed
        if (!timing_1.use(timing).add_to_boil) {
            return 0;
        }
        const AAU = alphaAcidUnits(units_1.convertMeasurableValue(amount, 'oz'), alpha_acid.value);
        const U = ibuUtilization(form, units_1.convertMeasurableValue(boilGravity, 'sg'), timing_1.boilTime(timing));
        return (U * AAU * 74.89) / units_1.convertMeasurableValue(postBoilVolume, 'gal');
    }));
    return {
        value: bitterness,
        unit: 'IBUs',
    };
};
exports.bitternessIbuTinseth = bitternessIbuTinseth;
// The preceived bitterness expressed in a ratio of IBUs to gravity. This is frequently seen expressed as BU/GU.
// The Gravity Units are the decimal portion of the original gravity
// http://beersmith.com/blog/2009/09/26/balancing-your-beer-with-the-bitterness-ratio/
const bitternessRatio = (ibu, gu) => ibu / gu;
exports.bitternessRatio = bitternessRatio;
// rager
const ragerHopGravityAdjustment = (sgb) => sgb <= 1.05 ? 0 : (sgb - 1.05) / 0.2;
const ragerUtil = (time) => 18.11 + 13.86 * Math.tanh((time - 31.32) / 18.27);
const ragerHopIbu = (hop, boilGravity, postBoilVolume) => {
    if (!timing_1.use(hop.timing).add_to_boil) {
        return 0;
    }
    const U = (ragerUtil(Math.floor(timing_1.boilTime(hop.timing) + 0.5)) *
        pelletFactor(hop.form)) /
        100;
    const AAU = alphaAcidUnits(hop.amount.value, hop.alpha_acid.value);
    return ((U * AAU * 74.89) /
        postBoilVolume.value /
        (1.0 + ragerHopGravityAdjustment(boilGravity.value)));
};
const bitternessIbuRager = (hops, boilGravity, postBoilVolume) => {
    const bitterness = utils_1.sum(hops.map((hop) => ragerHopIbu(hop, boilGravity, postBoilVolume)));
    return {
        value: bitterness,
        unit: 'IBUs',
    };
};
exports.bitternessIbuRager = bitternessIbuRager;
//# sourceMappingURL=hops.js.map